"use strict";
/* eslint-disable import/prefer-default-export */
/* A react hook async request */
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.useAsyncObjectManager = void 0;
const react_1 = require("react");
const prop_types_1 = __importDefault(require("prop-types"));
const merge_1 = __importDefault(require("lodash/merge"));
const async_hooks_use_async_request_1 = require("@react_db_client/async-hooks.use-async-request");
const react_db_client_helpers_generate_uid_1 = require("@samnbuk/react_db_client.helpers.generate-uid");
const helpers_1 = require("./helpers");
/**
 * Async React request hook
 *
 * @param {Object} {
 *   - `activeUid` {string} - the active object uid
 *   - `collection` {string} - the collection to load and save to
 *   - `isNew` {bool} - true if the object is new
 *   - `inputAdditionalData` {object} - additional data to be merged with the loaded data
 *   - `schema` {string|Array} - schema override for loading data
 *   - `loadOnInit` {bool} - true if we should load the object on component init
 * }
 * @return {Object} {
 *   - `saveData` {func} - save object function
 *   - `updateData` {func} - update edit data ()
 *   - `updateFormData` = (field, value, save = false) => {...}
 *   - `reload` {func} - reload and reset the object
 *   - `deleteObject` {func} - delete the object
 *   - `saveResponse` {string} - api response to saving the object
 *   - `deleteResponse` {func} - api response to deleting the object
 *   - `loadingData` {bool} - true when loading data
 *   - `savingData` {bool} - true when saving data
 *   - `deletingData` {bool} - true when deleting data
 *   - `data` {any} - the data returned from api and combined with additional data
 *   - `uid` {string} - the active uid
 *   - `callCount` {number} - number of times called
 *   - `hasLoaded` {bool} - true if we have loaded data
 * }
 */
const useAsyncObjectManager = ({ activeUid, collection, isNew: isNewIn = false, 
// TODO: Setting below causes it to rerun on render
inputAdditionalData = null, schema = 'all', populate = 'all', loadOnInit = true, reloadOnSave = false, onSavedCallback: onSavedCallbackIn = () => { } /* Returns a message string */, saveErrorCallback = () => { } /* Returns a AsyncRequestError */, onDeleteCallback = () => { }, asyncGetDocument, asyncPutDocument, asyncPostDocument, asyncDeleteDocument, }) => {
    const [isNew, setIsNew] = (0, react_1.useState)(!activeUid || isNewIn);
    const [uid] = (0, react_1.useState)(isNew || !activeUid ? (0, react_db_client_helpers_generate_uid_1.generateUid)(collection) : activeUid);
    const [editData, setEditData] = (0, react_1.useState)({});
    const [unsavedChanges, setUnsavedChanges] = (0, react_1.useState)(false); // TODO: Implement this
    const [resetToData, setResetToData] = (0, react_1.useState)({});
    const loadArgs = (0, react_1.useMemo)(() => [collection, uid, schema, populate], [collection, uid, schema, populate]);
    const [loadedData, setLoadedData] = (0, react_1.useState)(null);
    const loadedDataCallback = (newLoadedData) => {
        setLoadedData(newLoadedData);
        setEditData({});
    };
    const { call: loadAsync, loading: loadingData, callCount, hasLoaded, } = (0, async_hooks_use_async_request_1.useAsyncRequest)({
        id: 'loadAsync',
        // TODO: Add populate
        args: loadArgs,
        callFn: asyncGetDocument,
        callOnInit: loadOnInit && !isNew,
        callback: loadedDataCallback,
    });
    const combinedData = (0, react_1.useMemo)(() => {
        const _combinedData = (0, merge_1.default)(loadedData, inputAdditionalData, { uid }, editData);
        return _combinedData;
    }, [loadedData, inputAdditionalData, uid, editData]);
    /* Handle Saving Data */
    const onSavedCallback = (0, react_1.useCallback)((response) => {
        setIsNew(false);
        setUnsavedChanges(false);
        setResetToData(Object.assign(Object.assign(Object.assign(Object.assign({}, loadedData), inputAdditionalData), { uid }), editData));
        if (reloadOnSave) {
            loadAsync(loadArgs, (loadedResponse) => {
                loadedDataCallback(loadedResponse);
                onSavedCallbackIn(uid, response, combinedData);
            });
        }
        else {
            onSavedCallbackIn(uid, response, combinedData);
        }
    }, [
        uid,
        combinedData,
        onSavedCallbackIn,
        editData,
        inputAdditionalData,
        loadedData,
        reloadOnSave,
        loadAsync,
        loadArgs,
    ]);
    const { response: saveResponse, call: saveAsync, loading: savingData, } = (0, async_hooks_use_async_request_1.useAsyncRequest)({
        id: 'saveAsync',
        callFn: isNew ? asyncPostDocument : asyncPutDocument,
        callOnInit: false,
        callback: onSavedCallback,
        errorCallback: saveErrorCallback,
    });
    /* Handle Deleting Data */
    const { response: deleteResponse, call: deleteAsync, loading: deletingData, } = (0, async_hooks_use_async_request_1.useAsyncRequest)({
        id: 'deleteAsync',
        args: [collection, uid],
        callFn: isNew ? () => { } : asyncDeleteDocument,
        callOnInit: false,
    });
    // useEffect(() => {
    //   setResetToData({ ...loadedData, ...inputAdditionalData, uid });
    //   setEditData({ ...loadedData, ...inputAdditionalData, uid });
    //   // FIXME: Adding input additional data as as dependency causes a loop
    // }, [loadedData, inputAdditionalData, uid]);
    (0, react_1.useEffect)(() => {
        if (deleteResponse && deleteResponse.ok) {
            onDeleteCallback(uid, deleteResponse);
        }
    }, [deleteResponse, onDeleteCallback, uid]);
    /* Handle Update Data */
    const updateData = (0, react_1.useCallback)((newData) => {
        setUnsavedChanges(true);
        setEditData((prev) => (Object.assign(Object.assign({}, prev), newData)));
    }, []);
    const updateFormData = (0, react_1.useCallback)((field, value, save = false, nested = false) => {
        setUnsavedChanges(true);
        const saveAsyncInner = (newData) => __awaiter(void 0, void 0, void 0, function* () {
            const dataToSave = Object.assign(Object.assign(Object.assign(Object.assign({}, loadedData), inputAdditionalData), { uid }), newData);
            saveAsync([collection, uid, dataToSave]);
        });
        setEditData((prev) => {
            return (0, helpers_1.updateDict)(prev, saveAsyncInner)(field, value, save, nested);
        });
    }, [collection, uid, inputAdditionalData, loadedData, saveAsync]);
    /* UI Calls */
    const reload = (0, react_1.useCallback)(() => loadAsync(), [loadAsync]);
    const deleteObject = (0, react_1.useCallback)(() => deleteAsync(), [deleteAsync]);
    const resetData = (0, react_1.useCallback)(() => {
        setEditData(resetToData);
    }, [resetToData]);
    const saveData = (0, react_1.useCallback)(() => {
        saveAsync([collection, uid, combinedData]);
    }, [collection, uid, combinedData, saveAsync]);
    return {
        loadedData,
        saveData,
        updateData,
        updateFormData,
        resetData,
        reload,
        deleteObject,
        saveResponse,
        deleteResponse,
        loadingData,
        savingData,
        deletingData,
        data: combinedData,
        initialData: resetToData,
        uid,
        callCount,
        hasLoaded,
        unsavedChanges,
    };
};
exports.useAsyncObjectManager = useAsyncObjectManager;
exports.useAsyncObjectManager.propTypes = {
    activeUid: prop_types_1.default.string.isRequired,
    collection: prop_types_1.default.string.isRequired,
    isNew: prop_types_1.default.bool.isRequired,
    inputAdditionalData: prop_types_1.default.Object,
    schema: prop_types_1.default.arrayOf(prop_types_1.default.string).isRequired,
    loadOnInit: prop_types_1.default.bool,
    reloadOnSave: prop_types_1.default.bool,
    onSavedCallback: prop_types_1.default.func /* Returns a AsyncRequestError */,
    saveErrorCallback: prop_types_1.default.func.isRequired /* Returns a message string */,
    asyncGetDocument: prop_types_1.default.func.isRequired,
    asyncPutDocument: prop_types_1.default.func.isRequired,
    asyncPostDocument: prop_types_1.default.func.isRequired,
    asyncDeleteDocument: prop_types_1.default.func.isRequired,
};
//# sourceMappingURL=use-async-object-manager.js.map