"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
/* eslint-disable camelcase */
require("@samnbuk/react_db_client.helpers.enzyme-setup");
const react_hooks_1 = require("@testing-library/react-hooks");
const use_async_object_manager_1 = require("./use-async-object-manager");
const async_hooks_use_async_request_1 = require("@react_db_client/async-hooks.use-async-request");
const helpers_1 = require("./helpers");
Date.now = jest.fn().mockImplementation(() => 0);
jest.mock('@react_db_client/async-hooks.use-async-request', () => ({
    __esModule: true,
    useAsyncRequest: jest.fn(),
}));
const loadData = jest.fn();
// The default return values of the async hook
const loadedData = { loaded: 'data', nested: { hello: 'world' } };
const asyncHookReturnLoad_default = {
    call: loadData,
    loading: false,
    response: null,
};
const asyncHookReturnSave_default = {
    call: jest.fn().mockImplementation(() => { }),
    loading: false,
    response: null,
};
const asyncHookReturnSave_saved = {
    call: jest.fn().mockImplementation(() => { }),
    loading: false,
    response: { ok: true },
};
const asyncHookReturnDelete_default = {
    call: jest.fn().mockImplementation(() => { }),
    loading: false,
    response: null,
};
// async hook return values when state has changed
const asyncHookReturnLoad_loaded = {
    call: loadData,
    loading: false,
    response: loadedData,
};
const asyncGetDocument = jest.fn().mockImplementation(() => __awaiter(void 0, void 0, void 0, function* () { }));
const asyncPutDocument = jest.fn().mockImplementation(() => __awaiter(void 0, void 0, void 0, function* () { }));
const asyncPostDocument = jest.fn().mockImplementation(() => __awaiter(void 0, void 0, void 0, function* () { }));
const asyncDeleteDocument = jest.fn().mockImplementation(() => __awaiter(void 0, void 0, void 0, function* () { }));
const onSavedCallback = jest.fn();
const defaultArgs = {
    activeUid: 'demouid',
    collection: 'democollection',
    isNew: false,
    inputAdditionalData: {},
    schema: 'all',
    populate: 'all',
    loadOnInit: true,
    onSavedCallback,
    asyncGetDocument,
    asyncPutDocument,
    asyncPostDocument,
    asyncDeleteDocument,
};
const defaultArgsNewObject = {
    activeUid: null,
    collection: 'democollection',
    isNew: true,
    inputAdditionalData: {},
    schema: 'all',
    loadOnInit: true,
    onSavedCallback,
    asyncGetDocument,
    asyncPutDocument,
    asyncPostDocument,
    asyncDeleteDocument,
};
const setupAsyncRequestMock = ({ loadReturn, loadReturnLoaded, saveReturn, deleteReturn }) => ({ callFn, callOnInit }) => {
    switch (callFn) {
        case asyncGetDocument:
            return callOnInit ? loadReturnLoaded : loadReturn;
        case asyncPostDocument:
            return saveReturn;
        case asyncPutDocument:
            return saveReturn;
        case asyncDeleteDocument:
            return deleteReturn;
        default:
            return null;
    }
};
const defaultAsyncHookReturns = {
    loadReturn: asyncHookReturnLoad_default,
    loadReturnLoaded: asyncHookReturnLoad_loaded,
    saveReturn: asyncHookReturnSave_default,
    deleteReturn: asyncHookReturnDelete_default,
};
const loadDataAction = (rerender, args, demo_data) => __awaiter(void 0, void 0, void 0, function* () {
    rerender(args);
    const loadHookCall = async_hooks_use_async_request_1.useAsyncRequest.mock.calls[0].find((call) => call.id === 'loadAsync');
    (0, react_hooks_1.act)(() => {
        loadHookCall.callback(demo_data);
    });
});
describe('useAsyncObjectManager', () => {
    beforeEach(() => {
        loadData.mockClear();
        async_hooks_use_async_request_1.useAsyncRequest.mockClear();
        async_hooks_use_async_request_1.useAsyncRequest.mockImplementation(setupAsyncRequestMock(defaultAsyncHookReturns));
        onSavedCallback.mockClear();
        asyncHookReturnLoad_default.call.mockClear();
        asyncHookReturnLoad_loaded.call.mockClear();
        asyncHookReturnSave_default.call.mockClear();
        asyncHookReturnDelete_default.call.mockClear();
    });
    test('should initialize correct async request hooks', () => __awaiter(void 0, void 0, void 0, function* () {
        (0, react_hooks_1.renderHook)(() => (0, use_async_object_manager_1.useAsyncObjectManager)(defaultArgs));
        expect(async_hooks_use_async_request_1.useAsyncRequest).toHaveBeenCalledTimes(3);
        expect(async_hooks_use_async_request_1.useAsyncRequest.mock.calls).toEqual([
            // 1st call for load
            [
                {
                    id: 'loadAsync',
                    args: [
                        defaultArgs.collection,
                        defaultArgs.activeUid,
                        defaultArgs.schema,
                        defaultArgs.populate,
                    ],
                    callback: expect.any(Function),
                    callFn: asyncGetDocument,
                    callOnInit: true,
                },
            ],
            // second call for save
            [
                {
                    id: 'saveAsync',
                    callFn: asyncPutDocument,
                    callOnInit: false,
                    callback: expect.any(Function),
                    errorCallback: expect.any(Function),
                },
            ],
            // third call for delete
            [
                {
                    id: 'deleteAsync',
                    args: [defaultArgs.collection, defaultArgs.activeUid],
                    callFn: asyncDeleteDocument,
                    callOnInit: false,
                },
            ],
        ]);
    }));
    test('should pass load on init false to async request manager', () => __awaiter(void 0, void 0, void 0, function* () {
        (0, react_hooks_1.renderHook)(() => (0, use_async_object_manager_1.useAsyncObjectManager)(Object.assign(Object.assign({}, defaultArgs), { loadOnInit: false })));
        expect(async_hooks_use_async_request_1.useAsyncRequest).toHaveBeenCalledTimes(3);
        expect(async_hooks_use_async_request_1.useAsyncRequest.mock.calls[0]).toEqual([
            // 1st call for load
            {
                id: 'loadAsync',
                args: [
                    defaultArgs.collection,
                    defaultArgs.activeUid,
                    defaultArgs.schema,
                    defaultArgs.populate,
                ],
                callback: expect.any(Function),
                callFn: asyncGetDocument,
                callOnInit: false,
            },
        ]);
    }));
    /*  ========================== */
    describe('loading data', () => {
        test('should pass empty data before loaded by async hook', () => {
            const args = Object.assign(Object.assign({}, defaultArgs), { loadOnInit: false });
            const { result } = (0, react_hooks_1.renderHook)((props) => (0, use_async_object_manager_1.useAsyncObjectManager)(props), {
                initialProps: args,
            });
            expect(result.current.data).toEqual({
                uid: defaultArgs.activeUid,
            });
        });
        test('should call load data when we call the returned reload function', () => {
            const { result } = (0, react_hooks_1.renderHook)(() => (0, use_async_object_manager_1.useAsyncObjectManager)(defaultArgs));
            expect(asyncHookReturnLoad_loaded.call).not.toHaveBeenCalled();
            (0, react_hooks_1.act)(() => {
                result.current.reload();
            });
            expect(asyncHookReturnLoad_loaded.call).toHaveBeenCalled();
        });
        test('should pass loaded data from async hook to data returned value', () => __awaiter(void 0, void 0, void 0, function* () {
            const { result, rerender } = (0, react_hooks_1.renderHook)((props) => (0, use_async_object_manager_1.useAsyncObjectManager)(props), {
                initialProps: defaultArgs,
            });
            yield loadDataAction(rerender, defaultArgs, loadedData);
            expect(result.current.data).toEqual(Object.assign({ uid: defaultArgs.activeUid }, loadedData));
        }));
        test('should return combined data', () => __awaiter(void 0, void 0, void 0, function* () {
            const inputAdditionalData = { hello: 'world' };
            const args = Object.assign(Object.assign({}, defaultArgs), { inputAdditionalData });
            const { result, rerender } = (0, react_hooks_1.renderHook)((props) => (0, use_async_object_manager_1.useAsyncObjectManager)(props), {
                initialProps: args,
            });
            yield loadDataAction(rerender, args, loadedData);
            expect(result.current.data).toEqual(Object.assign(Object.assign({ uid: defaultArgs.activeUid }, inputAdditionalData), loadedData));
        }));
    });
    describe('Loaded', () => {
        let result;
        let rerender;
        beforeEach(() => __awaiter(void 0, void 0, void 0, function* () {
            ({ result, rerender } = (0, react_hooks_1.renderHook)((props) => (0, use_async_object_manager_1.useAsyncObjectManager)(props), {
                initialProps: defaultArgs,
            }));
            yield loadDataAction(rerender, defaultArgs, loadedData);
        }));
        describe('Updating data', () => {
            const updateData = { hello: 'world' };
            test('should update returned data after update data', () => __awaiter(void 0, void 0, void 0, function* () {
                yield loadDataAction(rerender, defaultArgs, loadedData);
                expect(result.current.unsavedChanges).not.toBeTruthy();
                expect(result.current.data).toEqual(Object.assign({ uid: defaultArgs.activeUid }, loadedData));
                (0, react_hooks_1.act)(() => {
                    result.current.updateData(updateData);
                });
                expect(result.current.unsavedChanges).toBeTruthy();
                expect(result.current.data).toEqual(Object.assign(Object.assign({ uid: defaultArgs.activeUid }, loadedData), updateData));
            }));
            test('should update returned data after update form data', () => __awaiter(void 0, void 0, void 0, function* () {
                const newData = { hello: 'new world' };
                expect(result.current.data).toEqual(Object.assign({ uid: defaultArgs.activeUid }, loadedData));
                (0, react_hooks_1.act)(() => {
                    result.current.updateFormData('hello', 'new world');
                });
                expect(result.current.data).toEqual(Object.assign(Object.assign({ uid: defaultArgs.activeUid }, loadedData), newData));
                expect(asyncHookReturnSave_default.call).not.toHaveBeenCalled();
            }));
            test('should call async save after update form data if save is true', () => __awaiter(void 0, void 0, void 0, function* () {
                const newData = { hello: 'new world' };
                expect(result.current.data).toEqual(Object.assign({ uid: defaultArgs.activeUid }, loadedData));
                (0, react_hooks_1.act)(() => {
                    result.current.updateFormData('hello', 'new world', true);
                });
                expect(result.current.data).toEqual(Object.assign(Object.assign({ uid: defaultArgs.activeUid }, loadedData), newData));
                expect(asyncHookReturnSave_default.call).toHaveBeenCalledWith([
                    defaultArgs.collection,
                    defaultArgs.activeUid,
                    Object.assign(Object.assign(Object.assign({}, loadedData), { uid: defaultArgs.activeUid }), newData),
                ]);
            }));
            test('should handle nested field values', () => __awaiter(void 0, void 0, void 0, function* () {
                /* Nested fields are split by dot notation. E.g. parent.child */
                const newData = { nested: { hello: 'new world' } };
                expect(result.current.data).toEqual(Object.assign({ uid: defaultArgs.activeUid }, loadedData));
                (0, react_hooks_1.act)(() => {
                    result.current.updateFormData('hello', 'new world', false, 'nested');
                });
                expect(result.current.data).toEqual(Object.assign(Object.assign({ uid: defaultArgs.activeUid }, loadedData), newData));
            }));
            test('should handle nested field value merge', () => __awaiter(void 0, void 0, void 0, function* () {
                /* Nested fields are split by dot notation. E.g. parent.child */
                const newData = { nested: { hello: 'new world', goodbye: 'old world' } };
                // act(() => {
                //   result.current.updateFormData('hello', 'new world', false, 'nested');
                // });
                (0, react_hooks_1.act)(() => {
                    result.current.updateFormData('goodbye', 'old world', false, 'nested');
                });
                expect(result.current.data).toEqual(Object.assign(Object.assign({ uid: defaultArgs.activeUid }, loadedData), newData));
            }));
        });
        describe('reloading', () => {
            beforeEach(() => {
                async_hooks_use_async_request_1.useAsyncRequest.mockReturnValue(asyncHookReturnLoad_default);
            });
            test('should call async reload when reload called', () => __awaiter(void 0, void 0, void 0, function* () {
                const { result } = (0, react_hooks_1.renderHook)(() => (0, use_async_object_manager_1.useAsyncObjectManager)(defaultArgs));
                expect(asyncHookReturnLoad_default.call).toHaveBeenCalledTimes(0);
                result.current.reload();
                expect(asyncHookReturnLoad_default.call).toHaveBeenCalledTimes(1);
            }));
        });
        describe('Deleting', () => {
            test('should call delete hook when delete func called', () => __awaiter(void 0, void 0, void 0, function* () {
                (0, react_hooks_1.act)(() => {
                    result.current.deleteObject();
                });
                expect(asyncHookReturnDelete_default.call).toHaveBeenCalledTimes(1);
            }));
        });
        /* ================ */
        describe('ResettingData', () => {
            const updateData = { hello: 'world' };
            // const resetAsync = () => {
            //   let i = 0;
            //   useAsyncRequest
            //     .mockImplementation(() => {
            //       // console.log(i)
            //       if (i % 3 === 0) return asyncHookReturnLoad_default;
            //       if (i % 3 === 1) return asyncHookReturnSave_default;
            //       if (i % 3 === 2) return asyncHookReturnDelete_default;
            //       i += 1;
            //       return null;
            //     });
            // };
            test('should reset returned data to loaded data', () => __awaiter(void 0, void 0, void 0, function* () {
                expect(result.current.data).toEqual(Object.assign({ uid: defaultArgs.activeUid }, loadedData));
                (0, react_hooks_1.act)(() => {
                    result.current.updateData(updateData);
                });
                expect(result.current.data).toEqual(Object.assign(Object.assign({ uid: defaultArgs.activeUid }, loadedData), updateData));
                (0, react_hooks_1.act)(() => {
                    result.current.resetData(updateData);
                });
                expect(result.current.data).toEqual(Object.assign({ uid: defaultArgs.activeUid }, loadedData));
            }));
            test('should reset back to last save point', () => {
                expect(result.current.data).toEqual(Object.assign({ uid: defaultArgs.activeUid }, loadedData));
                (0, react_hooks_1.act)(() => {
                    result.current.updateData(updateData);
                });
                expect(result.current.data).toEqual(Object.assign(Object.assign({ uid: defaultArgs.activeUid }, loadedData), updateData));
                async_hooks_use_async_request_1.useAsyncRequest
                    .mockReturnValueOnce(Object.assign(Object.assign({}, asyncHookReturnLoad_default), { response: loadedData }))
                    .mockReturnValueOnce(Object.assign(Object.assign({}, asyncHookReturnSave_default), { response: { ok: true } }))
                    .mockReturnValueOnce(asyncHookReturnDelete_default);
                const saveAsync = async_hooks_use_async_request_1.useAsyncRequest.mock.calls
                    .filter((c) => c[0].id === 'saveAsync')
                    .reverse()[0][0];
                (0, react_hooks_1.act)(() => {
                    saveAsync.callback('RESPONSE');
                });
                expect(result.current.data).toEqual(Object.assign(Object.assign({ uid: defaultArgs.activeUid }, loadedData), updateData));
                (0, react_hooks_1.act)(() => {
                    result.current.resetData();
                });
                expect(result.current.data).toEqual(Object.assign(Object.assign({ uid: defaultArgs.activeUid }, loadedData), updateData));
            });
        });
        describe('Saving Data', () => {
            test('should call async save data on save data fn call', () => __awaiter(void 0, void 0, void 0, function* () {
                result.current.saveData();
                expect(asyncHookReturnSave_default.call).toHaveBeenCalledWith([
                    defaultArgs.collection,
                    defaultArgs.activeUid,
                    Object.assign(Object.assign({}, loadedData), { uid: defaultArgs.activeUid }),
                ]);
            }));
            test('should call onsavedcallback after save', () => __awaiter(void 0, void 0, void 0, function* () {
                result.current.saveData();
                expect(loadData).not.toHaveBeenCalled();
                async_hooks_use_async_request_1.useAsyncRequest.mockImplementation(setupAsyncRequestMock(Object.assign(Object.assign({}, defaultAsyncHookReturns), { saveReturn: asyncHookReturnSave_saved })));
                const saveAsync = async_hooks_use_async_request_1.useAsyncRequest.mock.calls
                    .filter((c) => c[0].id === 'saveAsync')
                    .reverse()[0][0];
                (0, react_hooks_1.act)(() => {
                    saveAsync.callback({ ok: true });
                });
                expect(loadData).not.toHaveBeenCalled();
                expect(onSavedCallback).toHaveBeenCalledTimes(1);
                expect(onSavedCallback).toHaveBeenCalledWith(defaultArgs.activeUid, { ok: true }, Object.assign(Object.assign({}, loadedData), { uid: defaultArgs.activeUid }));
            }));
            // TODO: Fix below test
            // test('should not call onsavedcallback unless has saved', () => {
            //   const { result, rerender } = renderHook(() => useAsyncObjectManager(defaultArgs));
            //   result.current.saveData();
            //   useAsyncRequest.mockImplementation(
            //     setupAsyncRequestMock({ ...defaultAsyncHookReturns, saveReturn: asyncHookReturnSave_saved })
            //   );
            //   rerender();
            //   expect(onSavedCallback).not.toHaveBeenCalled();
            // });
        });
        describe('async state', () => {
            beforeEach(() => {
                async_hooks_use_async_request_1.useAsyncRequest
                    .mockReturnValueOnce(Object.assign(Object.assign({}, asyncHookReturnLoad_default), { loading: true }))
                    .mockReturnValueOnce(Object.assign(Object.assign({}, asyncHookReturnSave_default), { loading: true }))
                    .mockReturnValueOnce(Object.assign(Object.assign({}, asyncHookReturnDelete_default), { loading: true }))
                    .mockReturnValue(Object.assign(Object.assign({}, asyncHookReturnLoad_default), { loading: true }));
            });
            test('should pass loading state to hook out', () => {
                const { result } = (0, react_hooks_1.renderHook)(() => (0, use_async_object_manager_1.useAsyncObjectManager)(defaultArgs));
                expect(result.current.loadingData).toEqual(true);
                expect(result.current.savingData).toEqual(true);
                expect(result.current.deletingData).toEqual(true);
            });
        });
        describe('New Objects', () => {
            beforeEach(() => {
                async_hooks_use_async_request_1.useAsyncRequest.mockReturnValue(Object.assign(Object.assign({}, asyncHookReturnLoad_default), { loading: true }));
            });
            test('should assign a new uid to new objects', () => {
                const { result } = (0, react_hooks_1.renderHook)(() => (0, use_async_object_manager_1.useAsyncObjectManager)(defaultArgsNewObject));
                expect(result.current.data).toEqual({
                    uid: '0_democollection',
                });
            });
        });
    });
});
describe('Helpers', () => {
    describe('Update Dict', () => {
        test('simple field and value', () => {
            const initialDict = {};
            const field = 'hello';
            const value = 'world';
            const updatedDict = (0, helpers_1.updateDict)(initialDict)(field, value);
            const expectedDict = { hello: 'world' };
            expect(updatedDict).toEqual(expectedDict);
        });
        test('overwrite', () => {
            const initialDict = { hello: 'hi' };
            const field = 'hello';
            const value = 'world';
            const updatedDict = (0, helpers_1.updateDict)(initialDict)(field, value);
            const expectedDict = { hello: 'world' };
            expect(updatedDict).toEqual(expectedDict);
        });
        test('nested', () => {
            const initialDict = {};
            const field = 'hello';
            const value = 'world';
            const nested = 'parent';
            const updatedDict = (0, helpers_1.updateDict)(initialDict)(field, value, false, nested);
            const expectedDict = { parent: { hello: 'world' } };
            expect(updatedDict).toEqual(expectedDict);
        });
        test('nested override', () => {
            const initialDict = { parent: { hello: 'hi' } };
            const field = 'hello';
            const value = 'world';
            const nested = 'parent';
            const updatedDict = (0, helpers_1.updateDict)(initialDict)(field, value, false, nested);
            const expectedDict = { parent: { hello: 'world' } };
            expect(updatedDict).toEqual(expectedDict);
        });
        test('nested override merge', () => {
            const initialDict = { parent: { hello: 'hi', other: 'saveme' } };
            const field = 'hello';
            const value = 'world';
            const nested = 'parent';
            const updatedDict = (0, helpers_1.updateDict)(initialDict)(field, value, false, nested);
            const expectedDict = { parent: { hello: 'world', other: 'saveme' } };
            expect(updatedDict).toEqual(expectedDict);
        });
    });
});
//# sourceMappingURL=use-async-object-manager.test.js.map